<?php
// we have to use the postfix URL, because return is a keyword in php
namespace Makecommerce\Ecommerce\Controller\Checkout;

class Notify extends \Magento\Framework\App\Action\Action
{
    /**
     * Order processor
     * @var \Makecommerce\Ecommerce\Helper\OrderProcessor
     */
    protected $orderProcessor;

    /**
     * Validator ensures that the required fields exist and are in the correct format
     * @var \Makecommerce\Ecommerce\Helper\Validator $validator
     */
    protected $validator;

    /**
     * Logger logs into MAGENTOROOT/var/log/mk_ecommerce_$level.log
     * Example mk_ecommerce_info.log
     * @var \Makecommerce\Ecommerce\Helper\Validator $validator
     */
    protected $logger;

    /**
     * Notify constructor.
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Makecommerce\Ecommerce\Helper\Validator $validator
     * @param \Makecommerce\Ecommerce\Logger\Logger $logger
     * @param \Makecommerce\Ecommerce\Helper\OrderProcessor $orderProcessor
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Makecommerce\Ecommerce\Helper\Validator $validator,
        \Makecommerce\Ecommerce\Logger\Logger $logger,
        \Makecommerce\Ecommerce\Helper\OrderProcessor $orderProcessor
    ) {
        $this->orderProcessor = $orderProcessor;
        $this->validator = $validator;
        $this->logger = $logger;
        parent::__construct($context);
    }

    /**
     * Receives the return request from mk
     *
     * Redirect to checkout/onepage|multishipping/success, if the payment is successful.
     *
     * @return void
     */
    public function execute()
    {
        $transactionData = $this->getRequest()->getParams();

        $this->logger->info(
            'Transaction notify begin, incoming data:' .
            json_encode($transactionData, JSON_PRETTY_PRINT)
        );

        $this->validator->validate($transactionData);

        $transactionData = json_decode($transactionData['json'], true);

        $this->logger->info('Transaction notify data parsed:' . json_encode($transactionData, JSON_PRETTY_PRINT));

        $this->orderProcessor->process($transactionData);
    }
}
